@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-documents-panel.c,v 1.3 2006/12/10 17:02:05 paolo Exp $
+ * $Id: gedit-documents-panel.c 5512 2007-02-24 20:17:53Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -300,15 +300,22 @@ window_tab_added (GeditWindow         *window,
 	}
 	else
 	{
-		GtkTreeSelection *selection;
+		GeditTab *active_tab;
 
 		gtk_list_store_append (GTK_LIST_STORE (panel->priv->model), 
 				       &iter);
 
-		selection = gtk_tree_view_get_selection (
-					GTK_TREE_VIEW (panel->priv->treeview));
+		active_tab = gedit_window_get_active_tab (panel->priv->window);
 
-		gtk_tree_selection_select_iter (selection, &iter);
+		if (tab == active_tab)
+		{
+			GtkTreeSelection *selection;
+
+			selection = gtk_tree_view_get_selection (
+						GTK_TREE_VIEW (panel->priv->treeview));
+
+			gtk_tree_selection_select_iter (selection, &iter);
+		}
 	}
 
 	name = tab_get_name (tab);
