@@ -26,7 +26,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-prefs-manager-app.c 5645 2007-06-24 19:42:53Z pborelli $
+ * $Id: gedit-prefs-manager-app.c 5737 2007-07-23 16:32:51Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,8 +35,6 @@
 
 #include <string.h>
 
-#include <libgnome/gnome-config.h>
-
 #include "gedit-prefs-manager.h"
 #include "gedit-prefs-manager-private.h"
 #include "gedit-prefs-manager-app.h"
@@ -57,11 +55,6 @@ static void gedit_prefs_manager_system_font_changed	(GConfClient *client,
 							 GConfEntry  *entry,
 							 gpointer     user_data);
 
-static void gedit_prefs_manager_editor_colors_changed	(GConfClient *client,
-							 guint        cnxn_id,
-							 GConfEntry  *entry,
-							 gpointer     user_data);
-
 static void gedit_prefs_manager_tabs_size_changed	(GConfClient *client,
 							 guint        cnxn_id, 
 							 GConfEntry  *entry, 
@@ -92,6 +85,11 @@ static void gedit_prefs_manager_right_margin_changed	(GConfClient *client,
 							 GConfEntry  *entry, 
 							 gpointer     user_data);
 
+static void gedit_prefs_manager_smart_home_end_changed	(GConfClient *client,
+							 guint        cnxn_id, 
+							 GConfEntry  *entry, 
+							 gpointer     user_data);
+
 static void gedit_prefs_manager_hl_current_line_changed	(GConfClient *client,
 							 guint        cnxn_id, 
 							 GConfEntry  *entry, 
@@ -133,27 +131,471 @@ static void gedit_prefs_manager_lockdown_changed	(GConfClient *client,
 							 GConfEntry  *entry,
 							 gpointer     user_data);
 
+/* GUI state is serialized to a .desktop file, not in gconf */
+
+#define GEDIT_STATE_DEFAULT_WINDOW_STATE	0
+#define GEDIT_STATE_DEFAULT_WINDOW_WIDTH	650
+#define GEDIT_STATE_DEFAULT_WINDOW_HEIGHT	500
+#define GEDIT_STATE_DEFAULT_SIDE_PANEL_SIZE	200
+#define GEDIT_STATE_DEFAULT_BOTTOM_PANEL_SIZE	140
+
+#define GEDIT_STATE_FILE_LOCATION ".gnome2/gedit-2"
+
+#define GEDIT_STATE_WINDOW_GROUP "window"
+#define GEDIT_STATE_WINDOW_STATE "state"
+#define GEDIT_STATE_WINDOW_HEIGHT "height"
+#define GEDIT_STATE_WINDOW_WIDTH "width"
+#define GEDIT_STATE_SIDE_PANEL_SIZE "side_panel_size"
+#define GEDIT_STATE_BOTTOM_PANEL_SIZE "bottom_panel_size"
+#define GEDIT_STATE_SIDE_PANEL_ACTIVE_PAGE "side_panel_active_page"
+#define GEDIT_STATE_BOTTOM_PANEL_ACTIVE_PAGE "bottom_panel_active_page"
+
+#define GEDIT_STATE_FILEFILTER_GROUP "filefilter"
+#define GEDIT_STATE_FILEFILTER_ID "id"
 
 static gint window_state = -1;
 static gint window_height = -1;
 static gint window_width = -1;
 static gint side_panel_size = -1;
 static gint bottom_panel_size = -1;
-
 static gint side_panel_active_page = 0;
 static gint bottom_panel_active_page = 0;
-
 static gint active_file_filter = -1;
 
+static GKeyFile *
+get_gedit_state_file ()
+{
+	static GKeyFile *state_file = NULL;
+
+	if (state_file == NULL)
+	{
+		const gchar *home;
+		gchar *path;
+		GError *err = NULL;
+
+		state_file = g_key_file_new ();
+
+		home = g_get_home_dir ();
+		if (home == NULL)
+		{
+			g_warning ("Could not get HOME directory\n");
+			goto out;
+		}
+
+		path = g_build_filename (home,
+					 GEDIT_STATE_FILE_LOCATION,
+					 NULL);
+
+		if (!g_key_file_load_from_file (state_file,
+						path,
+						G_KEY_FILE_NONE,
+						&err))
+		{
+			if (err->domain != G_FILE_ERROR ||
+			    err->code != G_FILE_ERROR_NOENT)
+			{
+				g_warning ("Could not load gedit state file: %s\n",
+					   err->message);
+			}
+		}
+
+		g_free (path);
+	}
+
+ out:
+	g_return_val_if_fail (state_file != NULL, NULL);
+	return state_file;
+}
+
+static void
+gedit_state_get_int (const gchar *group,
+		     const gchar *key,
+		     gint         defval,
+		     gint        *result)
+{
+	GKeyFile *state_file;
+	gint res;
+	GError *err = NULL;
+
+	state_file = get_gedit_state_file ();
+	res = g_key_file_get_integer (state_file,
+				      group,
+				      key,
+				      &err);
+
+	if (err != NULL)
+	{
+		if ((err->domain != G_KEY_FILE_ERROR) ||
+		    ((err->code != G_KEY_FILE_ERROR_GROUP_NOT_FOUND &&
+		      err->code != G_KEY_FILE_ERROR_KEY_NOT_FOUND)))
+		{
+			g_warning ("Could not get state value %s::%s : %s\n",
+				   group,
+				   key,
+				   err->message);
+		}
+
+		if (defval > 0)
+			*result = defval;
+	}
+	else
+	{
+		*result = res;
+	}
+}
+
+static void
+gedit_state_set_int (const gchar *group,
+		     const gchar *key,
+		     gint         value)
+{
+	GKeyFile *state_file;
+
+	state_file = get_gedit_state_file ();
+	g_key_file_set_integer (state_file,
+				group,
+				key,
+				value);
+}
+
+static gboolean
+gedit_state_file_sync ()
+{
+	GKeyFile *state_file;
+	const gchar *home;
+	gchar *path;
+	gchar *content;
+	gsize length;
+	GError *err = NULL;
+	gboolean ret = FALSE;
+
+	state_file = get_gedit_state_file ();
+	g_return_val_if_fail (state_file != NULL, FALSE);
+
+	home = g_get_home_dir ();
+	if (home == NULL)
+	{
+		g_warning ("Could not get HOME directory\n");
+		return ret;
+	}
+
+	path = g_build_filename (home,
+				 GEDIT_STATE_FILE_LOCATION,
+				 NULL);
+
+	content = g_key_file_to_data (state_file,
+				      &length,
+				      &err);
+
+	if (err != NULL)
+	{
+		g_warning ("Could not get data from state file: %s\n",
+			   err->message);
+		goto out;
+	}
+
+	if ((content != NULL) &&
+	    (!g_file_set_contents (path,
+				   content,
+				   length,
+				   &err)))
+	{
+		g_warning ("Could not write gedit state file: %s\n",
+			   err->message);
+		goto out;
+	}
+
+	ret = TRUE;
+
+ out:
+	g_free (content);
+	g_free (path);
+
+	return TRUE;
+}
+
+/* Window state */
+gint
+gedit_prefs_manager_get_window_state (void)
+{
+	if (window_state == -1)
+	{
+		gedit_state_get_int (GEDIT_STATE_WINDOW_GROUP,
+				     GEDIT_STATE_WINDOW_STATE,
+				     GEDIT_STATE_DEFAULT_WINDOW_STATE,
+				     &window_state);
+	}
+
+	return window_state;
+}
+			
+void
+gedit_prefs_manager_set_window_state (gint ws)
+{
+	g_return_if_fail (ws > -1);
+	
+	window_state = ws;
+
+	gedit_state_set_int (GEDIT_STATE_WINDOW_GROUP,
+			     GEDIT_STATE_WINDOW_STATE,
+			     ws);
+}
+
+gboolean
+gedit_prefs_manager_window_state_can_set (void)
+{
+	return TRUE;
+}
+
+/* Window size */
+void
+gedit_prefs_manager_get_window_size (gint *width, gint *height)
+{
+	g_return_if_fail (width != NULL && height != NULL);
+
+	if (window_width == -1)
+	{
+		gedit_state_get_int (GEDIT_STATE_WINDOW_GROUP,
+				     GEDIT_STATE_WINDOW_WIDTH,
+				     GEDIT_STATE_DEFAULT_WINDOW_WIDTH,
+				     &window_width);
+	}
+
+	if (window_height == -1)
+	{
+		gedit_state_get_int (GEDIT_STATE_WINDOW_GROUP,
+				     GEDIT_STATE_WINDOW_HEIGHT,
+				     GEDIT_STATE_DEFAULT_WINDOW_HEIGHT,
+				     &window_height);
+	}
+
+	*width = window_width;
+	*height = window_height;
+}
+
+void
+gedit_prefs_manager_get_default_window_size (gint *width, gint *height)
+{
+	g_return_if_fail (width != NULL && height != NULL);
+
+	*width = GEDIT_STATE_DEFAULT_WINDOW_WIDTH;
+	*height = GEDIT_STATE_DEFAULT_WINDOW_HEIGHT;
+}
+
+void
+gedit_prefs_manager_set_window_size (gint width, gint height)
+{
+	g_return_if_fail (width > -1 && height > -1);
+
+	window_width = width;
+	window_height = height;
+
+	gedit_state_set_int (GEDIT_STATE_WINDOW_GROUP,
+			     GEDIT_STATE_WINDOW_WIDTH,
+			     width);
+	gedit_state_set_int (GEDIT_STATE_WINDOW_GROUP,
+			     GEDIT_STATE_WINDOW_HEIGHT,
+			     height);
+}
+
+gboolean 
+gedit_prefs_manager_window_size_can_set (void)
+{
+	return TRUE;
+}
+
+/* Side panel */
+gint
+gedit_prefs_manager_get_side_panel_size (void)
+{
+	if (side_panel_size == -1)
+	{
+		gedit_state_get_int (GEDIT_STATE_WINDOW_GROUP,
+				     GEDIT_STATE_SIDE_PANEL_SIZE,
+				     GEDIT_STATE_DEFAULT_SIDE_PANEL_SIZE,
+				     &side_panel_size);
+	}
+
+	return side_panel_size;
+}
+
+gint 
+gedit_prefs_manager_get_default_side_panel_size (void)
+{
+	return GEDIT_STATE_DEFAULT_SIDE_PANEL_SIZE;
+}
+
+void 
+gedit_prefs_manager_set_side_panel_size (gint ps)
+{
+	g_return_if_fail (ps > -1);
+	
+	if (side_panel_size == ps)
+		return;
+		
+	side_panel_size = ps;
+	gedit_state_set_int (GEDIT_STATE_WINDOW_GROUP,
+			     GEDIT_STATE_SIDE_PANEL_SIZE,
+			     ps);
+}
+
+gboolean 
+gedit_prefs_manager_side_panel_size_can_set (void)
+{
+	return TRUE;
+}
+
+gint
+gedit_prefs_manager_get_side_panel_active_page (void)
+{
+	if (side_panel_active_page == 0)
+	{
+		gedit_state_get_int (GEDIT_STATE_WINDOW_GROUP,
+				     GEDIT_STATE_SIDE_PANEL_ACTIVE_PAGE,
+				     -1,
+				     &side_panel_active_page);
+	}
+
+	return side_panel_active_page;
+}
+
+void
+gedit_prefs_manager_set_side_panel_active_page (gint id)
+{
+	if (side_panel_active_page == id)
+		return;
+
+	side_panel_active_page = id;
+	gedit_state_set_int (GEDIT_STATE_WINDOW_GROUP,
+			     GEDIT_STATE_SIDE_PANEL_ACTIVE_PAGE,
+			     id);
+}
+
+gboolean 
+gedit_prefs_manager_side_panel_active_page_can_set (void)
+{
+	return TRUE;
+}
+
+/* Bottom panel */
+gint
+gedit_prefs_manager_get_bottom_panel_size (void)
+{
+	if (bottom_panel_size == -1)
+	{
+		gedit_state_get_int (GEDIT_STATE_WINDOW_GROUP,
+				     GEDIT_STATE_BOTTOM_PANEL_SIZE,
+				     GEDIT_STATE_DEFAULT_BOTTOM_PANEL_SIZE,
+				     &bottom_panel_size);
+	}
+
+	return bottom_panel_size;
+}
+
+gint 
+gedit_prefs_manager_get_default_bottom_panel_size (void)
+{
+	return GEDIT_STATE_DEFAULT_BOTTOM_PANEL_SIZE;
+}
+
+void 
+gedit_prefs_manager_set_bottom_panel_size (gint ps)
+{
+	g_return_if_fail (ps > -1);
+
+	if (bottom_panel_size == ps)
+		return;
+	
+	bottom_panel_size = ps;
+	gedit_state_set_int (GEDIT_STATE_WINDOW_GROUP,
+			     GEDIT_STATE_BOTTOM_PANEL_SIZE,
+			     ps);
+}
+
+gboolean 
+gedit_prefs_manager_bottom_panel_size_can_set (void)
+{
+	return TRUE;
+}
+
+gint
+gedit_prefs_manager_get_bottom_panel_active_page (void)
+{
+	if (bottom_panel_active_page == 0)
+	{
+		gedit_state_get_int (GEDIT_STATE_WINDOW_GROUP,
+				     GEDIT_STATE_BOTTOM_PANEL_ACTIVE_PAGE,
+				     -1,
+				     &bottom_panel_active_page);
+	}
+
+	return bottom_panel_active_page;
+}
+
+void
+gedit_prefs_manager_set_bottom_panel_active_page (gint id)
+{
+	if (bottom_panel_active_page == id)
+		return;
+
+	bottom_panel_active_page = id;
+	gedit_state_set_int (GEDIT_STATE_WINDOW_GROUP,
+			     GEDIT_STATE_BOTTOM_PANEL_ACTIVE_PAGE,
+			     id);
+}
+
+gboolean 
+gedit_prefs_manager_bottom_panel_active_page_can_set (void)
+{
+	return TRUE;
+}
+
+/* File filter */
+gint
+gedit_prefs_manager_get_active_file_filter (void)
+{
+	if (active_file_filter == -1)
+	{
+		gedit_state_get_int (GEDIT_STATE_FILEFILTER_GROUP,
+				     GEDIT_STATE_FILEFILTER_ID,
+				     0,
+				     &active_file_filter);
+	}
+
+	return active_file_filter;
+}
+
+void
+gedit_prefs_manager_set_active_file_filter (gint id)
+{
+	g_return_if_fail (id >= 0);
+	
+	if (active_file_filter == id)
+		return;
+
+	active_file_filter = id;
+	gedit_state_set_int (GEDIT_STATE_FILEFILTER_GROUP,
+			     GEDIT_STATE_FILEFILTER_ID,
+			     id);
+}
+
+gboolean 
+gedit_prefs_manager_active_file_filter_can_set (void)
+{
+	return TRUE;
+}
+
+/* Normal prefs are stored in GConf */
+
 gboolean
 gedit_prefs_manager_app_init (void)
 {
 	gedit_debug (DEBUG_PREFS);
 
 	g_return_val_if_fail (gedit_prefs_manager == NULL, FALSE);
-	
+
 	gedit_prefs_manager_init ();
-	
+
 	if (gedit_prefs_manager != NULL)
 	{
 		/* TODO: notify, add dirs */
@@ -177,11 +619,6 @@ gedit_prefs_manager_app_init (void)
 				gedit_prefs_manager_system_font_changed,
 				NULL, NULL, NULL);
 
-		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
-				GPM_COLORS_DIR,
-				gedit_prefs_manager_editor_colors_changed,
-				NULL, NULL, NULL);
-
 		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
 				GPM_TABS_DIR,
 				gedit_prefs_manager_tabs_size_changed,
@@ -212,6 +649,11 @@ gedit_prefs_manager_app_init (void)
 				gedit_prefs_manager_right_margin_changed,
 				NULL, NULL, NULL);
 
+		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
+				GPM_SMART_HOME_END_DIR,
+				gedit_prefs_manager_smart_home_end_changed,
+				NULL, NULL, NULL);
+
 		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
 				GPM_CURRENT_LINE_DIR,
 				gedit_prefs_manager_hl_current_line_changed,
@@ -264,224 +706,10 @@ gedit_prefs_manager_app_shutdown ()
 
 	gedit_prefs_manager_shutdown ();
 
-	gnome_config_sync ();
+	gedit_state_file_sync ();
 }
 
-/* Window state */
-gint
-gedit_prefs_manager_get_window_state (void)
-{
-	if (window_state == -1)
-		window_state = gnome_config_get_int (GPM_WINDOW_STATE "=" GPM_DEFAULT_WINDOW_STATE_STR);
 
-	return window_state;
-}
-			
-void
-gedit_prefs_manager_set_window_state (gint ws)
-{
-	g_return_if_fail (ws > -1);
-	
-	window_state = ws;
-	gnome_config_set_int (GPM_WINDOW_STATE, ws);
-}
-
-gboolean
-gedit_prefs_manager_window_state_can_set (void)
-{
-	return TRUE;
-}
-
-/* Window size */
-void
-gedit_prefs_manager_get_window_size (gint *width, gint *height)
-{
-	g_return_if_fail (width != NULL && height != NULL);
-
-	if (window_width == -1)
-		window_width = gnome_config_get_int (GPM_WINDOW_WIDTH "=" GPM_DEFAULT_WINDOW_WIDTH_STR);
-
-	if (window_height == -1)
-		window_height = gnome_config_get_int (GPM_WINDOW_HEIGHT "=" GPM_DEFAULT_WINDOW_HEIGHT_STR);
-
-	*width = window_width;
-	*height = window_height;
-}
-
-void
-gedit_prefs_manager_get_default_window_size (gint *width, gint *height)
-{
-	g_return_if_fail (width != NULL && height != NULL);
-
-	*width = GPM_DEFAULT_WINDOW_WIDTH;
-	*height = GPM_DEFAULT_WINDOW_HEIGHT;
-}
-
-void
-gedit_prefs_manager_set_window_size (gint width, gint height)
-{
-	g_return_if_fail (width > -1 && height > -1);
-
-	window_width = width;
-	window_height = height;
-
-	gnome_config_set_int (GPM_WINDOW_WIDTH, width);
-	gnome_config_set_int (GPM_WINDOW_HEIGHT, height);
-}
-
-gboolean 
-gedit_prefs_manager_window_size_can_set (void)
-{
-	return TRUE;
-}
-
-/* Side panel */
-gint
-gedit_prefs_manager_get_side_panel_size (void)
-{
-	if (side_panel_size == -1)
-		side_panel_size = gnome_config_get_int (GPM_SIDE_PANEL_SIZE "=" GPM_DEFAULT_SIDE_PANEL_SIZE_STR);
-
-	return side_panel_size;
-}
-
-gint 
-gedit_prefs_manager_get_default_side_panel_size (void)
-{
-	return GPM_DEFAULT_SIDE_PANEL_SIZE;
-}
-
-void 
-gedit_prefs_manager_set_side_panel_size (gint ps)
-{
-	g_return_if_fail (ps > -1);
-	
-	if (side_panel_size == ps)
-		return;
-		
-	side_panel_size = ps;
-	gnome_config_set_int (GPM_SIDE_PANEL_SIZE, ps);
-}
-
-gboolean 
-gedit_prefs_manager_side_panel_size_can_set (void)
-{
-	return TRUE;
-}
-
-gint
-gedit_prefs_manager_get_side_panel_active_page (void)
-{
-	if (side_panel_active_page == 0)
-		side_panel_active_page = gnome_config_get_int (
-				GPM_SIDE_PANEL_ACTIVE_PAGE);
-
-	return side_panel_active_page;
-}
-
-void
-gedit_prefs_manager_set_side_panel_active_page (gint id)
-{
-	if (side_panel_active_page == id)
-		return;
-
-	side_panel_active_page = id;
-	gnome_config_set_int (GPM_SIDE_PANEL_ACTIVE_PAGE, id);
-}
-
-gboolean 
-gedit_prefs_manager_side_panel_active_page_can_set (void)
-{
-	return TRUE;
-}
-
-/* Bottom panel */
-gint
-gedit_prefs_manager_get_bottom_panel_size (void)
-{
-	if (bottom_panel_size == -1)
-		bottom_panel_size = gnome_config_get_int (GPM_BOTTOM_PANEL_SIZE "=" GPM_DEFAULT_BOTTOM_PANEL_SIZE_STR);
-
-	return bottom_panel_size;
-}
-
-gint 
-gedit_prefs_manager_get_default_bottom_panel_size (void)
-{
-	return GPM_DEFAULT_BOTTOM_PANEL_SIZE;
-}
-
-void 
-gedit_prefs_manager_set_bottom_panel_size (gint ps)
-{
-	g_return_if_fail (ps > -1);
-
-	if (bottom_panel_size == ps)
-		return;
-	
-	bottom_panel_size = ps;
-	gnome_config_set_int (GPM_BOTTOM_PANEL_SIZE, ps);
-}
-
-gboolean 
-gedit_prefs_manager_bottom_panel_size_can_set (void)
-{
-	return TRUE;
-}
-
-gint
-gedit_prefs_manager_get_bottom_panel_active_page (void)
-{
-	if (bottom_panel_active_page == 0)
-		bottom_panel_active_page = gnome_config_get_int (
-				GPM_BOTTOM_PANEL_ACTIVE_PAGE);
-
-	return bottom_panel_active_page;
-}
-
-void
-gedit_prefs_manager_set_bottom_panel_active_page (gint id)
-{
-	if (bottom_panel_active_page == id)
-		return;
-
-	bottom_panel_active_page = id;
-	gnome_config_set_int (GPM_BOTTOM_PANEL_ACTIVE_PAGE, id);
-}
-
-gboolean 
-gedit_prefs_manager_bottom_panel_active_page_can_set (void)
-{
-	return TRUE;
-}
-
-/* File filter */
-gint
-gedit_prefs_manager_get_active_file_filter (void)
-{
-	if (active_file_filter == -1)
-		active_file_filter = gnome_config_get_int (GPM_ACTIVE_FILE_FILTER "=0");
-
-	return active_file_filter;
-}
-
-void
-gedit_prefs_manager_set_active_file_filter (gint id)
-{
-	g_return_if_fail (id >= 0);
-	
-	if (active_file_filter == id)
-		return;
-
-	active_file_filter = id;
-	gnome_config_set_int (GPM_ACTIVE_FILE_FILTER, id);
-}
-
-gboolean 
-gedit_prefs_manager_active_file_filter_can_set (void)
-{
-	return TRUE;
-}
 static void 
 gedit_prefs_manager_editor_font_changed (GConfClient *client,
 					 guint        cnxn_id, 
@@ -588,145 +816,6 @@ gedit_prefs_manager_system_font_changed (GConfClient *client,
 	g_free (font);
 }
 
-static void 
-set_colors (gboolean  def, 
-	    GdkColor *backgroud, 
-	    GdkColor *text,
-	    GdkColor *selection, 
-	    GdkColor *sel_text)
-{
-	GList *views;
-	GList *l;
-
-	views = gedit_app_get_views (gedit_app_get_default ());
-	l = views;
-
-	while (l != NULL)
-	{
-		gedit_view_set_colors (GEDIT_VIEW (l->data),
-				       def,
-				       backgroud,
-				       text,
-				       selection,
-				       sel_text);
-
-		l = l->next;
-	}
-
-	g_list_free (views);
-}
-
-static void 
-gedit_prefs_manager_editor_colors_changed (GConfClient *client,
-					   guint        cnxn_id, 
-					   GConfEntry  *entry, 
-					   gpointer     user_data)
-{	
-	gchar *str_color;
-	GdkColor color;
-
-	gedit_debug (DEBUG_PREFS);
-
-	g_return_if_fail (entry->key != NULL);
-	g_return_if_fail (entry->value != NULL);
-
-	if (strcmp (entry->key, GPM_USE_DEFAULT_COLORS) == 0)
-	{
-		gboolean def = TRUE;
-		
-		if (entry->value->type == GCONF_VALUE_BOOL)
-			def = gconf_value_get_bool (entry->value);
-		else
-			def = GPM_DEFAULT_USE_DEFAULT_COLORS;
-
-		if (def)
-			set_colors (TRUE, NULL, NULL, NULL, NULL);
-		else
-		{
-			GdkColor background, text, selection, sel_text;
-
-			background = gedit_prefs_manager_get_background_color ();
-			text = gedit_prefs_manager_get_text_color ();
-			selection = gedit_prefs_manager_get_selection_color ();
-			sel_text = gedit_prefs_manager_get_selected_text_color ();
-
-			set_colors (FALSE,
-				    &background, 
-				    &text, 
-				    &selection, 
-				    &sel_text);
-		}
-
-		return;
-	}
-	
-	if (gedit_prefs_manager_get_use_default_colors ())
-	{
-		set_colors (TRUE, NULL, NULL, NULL, NULL);
-		return;
-	}
-		
-	if (strcmp (entry->key, GPM_BACKGROUND_COLOR) == 0)
-	{
-		if (entry->value->type == GCONF_VALUE_STRING)
-			str_color = g_strdup (gconf_value_get_string (entry->value));
-		else
-			str_color = g_strdup (GPM_DEFAULT_BACKGROUND_COLOR);
-				
-		gdk_color_parse (str_color, &color);	
-		g_free (str_color);
-
-		set_colors (FALSE, &color, NULL, NULL, NULL);
-	
-		return;
-	}
-
-	if (strcmp (entry->key, GPM_TEXT_COLOR) == 0)
-	{
-		if (entry->value->type == GCONF_VALUE_STRING)
-			str_color = g_strdup (gconf_value_get_string (entry->value));
-		else
-			str_color = g_strdup (GPM_DEFAULT_TEXT_COLOR);
-				
-		gdk_color_parse (str_color, &color);	
-		g_free (str_color);
-
-		set_colors (FALSE, NULL, &color, NULL, NULL);
-	
-		return;
-	}
-
-	if (strcmp (entry->key, GPM_SELECTION_COLOR) == 0)
-	{
-		if (entry->value->type == GCONF_VALUE_STRING)
-			str_color = g_strdup (gconf_value_get_string (entry->value));
-		else
-			str_color = g_strdup (GPM_DEFAULT_SELECTION_COLOR);
-				
-		gdk_color_parse (str_color, &color);	
-		g_free (str_color);
-
-		set_colors (FALSE, NULL, NULL, &color, NULL);
-	
-		return;
-	}
-
-	if (strcmp (entry->key, GPM_SELECTED_TEXT_COLOR) == 0)
-	{
-		if (entry->value->type == GCONF_VALUE_STRING)
-			str_color = g_strdup (gconf_value_get_string (entry->value));
-		else
-			str_color = g_strdup (GPM_DEFAULT_SELECTED_TEXT_COLOR);
-
-		gdk_color_parse (str_color, &color);	
-		g_free (str_color);
-
-		set_colors (FALSE, NULL, NULL, NULL, &color);
-	
-		return;
-	}
-}
-
 static void 
 gedit_prefs_manager_tabs_size_changed (GConfClient *client,
 				       guint        cnxn_id,
@@ -1099,6 +1188,63 @@ gedit_prefs_manager_right_margin_changed (GConfClient *client,
 	}
 }
 
+static GtkSourceSmartHomeEndType
+get_smart_home_end_from_string (const gchar *str)
+{
+	GtkSourceSmartHomeEndType res;
+
+	g_return_val_if_fail (str != NULL, GTK_SOURCE_SMART_HOME_END_AFTER);
+
+	if (strcmp (str, "DISABLED") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_DISABLED;
+	else if (strcmp (str, "BEFORE") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_BEFORE;
+	else if (strcmp (str, "ALWAYS") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_ALWAYS;
+	else
+		res = GTK_SOURCE_SMART_HOME_END_AFTER;
+
+	return res;
+}
+
+static void
+gedit_prefs_manager_smart_home_end_changed (GConfClient *client,
+					    guint        cnxn_id,
+					    GConfEntry  *entry,
+					    gpointer     user_data)
+{
+	gedit_debug (DEBUG_PREFS);
+
+	g_return_if_fail (entry->key != NULL);
+	g_return_if_fail (entry->value != NULL);
+
+	if (strcmp (entry->key, GPM_SMART_HOME_END) == 0)
+	{
+		GtkSourceSmartHomeEndType smart_he;
+		GList *views;
+		GList *l;
+
+		if (entry->value->type == GCONF_VALUE_STRING)
+			smart_he = 
+				get_smart_home_end_from_string (gconf_value_get_string (entry->value));	
+		else
+			smart_he = get_smart_home_end_from_string (GPM_DEFAULT_SMART_HOME_END);
+
+		views = gedit_app_get_views (gedit_app_get_default ());
+		l = views;
+
+		while (l != NULL)
+		{
+			gtk_source_view_set_smart_home_end (GTK_SOURCE_VIEW (l->data),
+							    smart_he);
+
+			l = l->next;
+		}
+
+		g_list_free (views);
+	}
+}
+
 static void
 gedit_prefs_manager_syntax_hl_enable_changed (GConfClient *client,
 					      guint        cnxn_id,
