@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-utils.c,v 1.79 2006/08/24 13:17:13 paolo Exp $
+ * $Id: gedit-utils.c 5742 2007-07-25 14:02:26Z jessevdk $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,7 +49,6 @@
 #include <glib/gi18n.h>
 #include <glade/glade-xml.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <libgnome/gnome-url.h>
 
 #include "gedit-utils.h"
 
@@ -189,6 +188,16 @@ gedit_utils_menu_position_under_tree_view (GtkMenu  *menu,
 	}
 }
 
+/* FIXME: remove this with gtk 2.12, it has gdk_color_to_string */
+gchar * 
+gedit_gdk_color_to_string (GdkColor color)
+{
+	return g_strdup_printf ("#%04x%04x%04x",
+				color.red, 
+				color.green,
+				color.blue);
+}
+
 GtkWidget *
 gedit_gtk_button_new_with_stock_icon (const gchar *label,
 				      const gchar *stock_id)
@@ -883,7 +892,7 @@ gedit_utils_activate_url (GtkAboutDialog *about,
 			  const gchar    *url,
 			  gpointer        data)
 {
-	gnome_url_show (url, NULL);
+	gnome_vfs_url_show (url);
 }
 
 static gboolean
@@ -1130,3 +1139,45 @@ gedit_utils_format_uri_for_display (const gchar *uri)
 		return uri_for_display;
 	}
 }
+
+/**
+ * gedit_utils_drop_get_uris:
+ * @selection_data: the #GtkSelectionData from drag_data_received
+ * @info: the info from drag_data_received
+ *
+ * Create a list of valid uri's from a uri-list drop.
+ * 
+ * Return value: a string array which will hold the uris or NULL if there 
+ *		 were no valid uris. g_strfreev should be used when the 
+ *		 string array is no longer used
+ */
+gchar **
+gedit_utils_drop_get_uris (GtkSelectionData *selection_data)
+{
+	gchar **uris;
+	gint i;
+	gint p = 0;
+	gchar **uri_list;
+
+	uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
+	uri_list = g_new0(gchar *, g_strv_length (uris) + 1);
+
+	for (i = 0; uris[i] != NULL; i++)
+	{
+		gchar *uri;
+		
+		uri = gedit_utils_make_canonical_uri_from_shell_arg (uris[i]);
+		
+		/* Silently ignore malformed URI/filename */
+		if (uri != NULL)
+			uri_list[p++] = uri;
+	}
+
+	if (*uri_list == NULL)
+	{
+		g_free(uri_list);
+		return NULL;
+	}
+
+	return uri_list;
+}
