@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-utils.h,v 1.37.2.1 2006/04/20 16:53:41 pborelli Exp $
+ * $Id: gedit-utils.h 5742 2007-07-25 14:02:26Z jessevdk $
  */
 
 #ifndef __GEDIT_UTILS_H__
@@ -61,6 +61,15 @@ void		 gedit_utils_menu_position_under_widget (GtkMenu  *menu,
 							 gboolean *push_in,
 							 gpointer  user_data);
 
+void		 gedit_utils_menu_position_under_tree_view
+							(GtkMenu  *menu,
+							 gint     *x,
+							 gint     *y,
+							 gboolean *push_in,
+							 gpointer  user_data);
+
+gchar		*gedit_gdk_color_to_string		(GdkColor color);
+
 GtkWidget	*gedit_gtk_button_new_with_stock_icon	(const gchar *label,
 							 const gchar *stock_id);
 
@@ -137,6 +146,9 @@ gchar		*gedit_utils_make_canonical_uri_from_shell_arg
  * resulting string */
 gchar		*gedit_utils_format_uri_for_display 	(const gchar *uri);
 
+/* Turns data from a drop into a list of well formatted uris */
+gchar 	       **gedit_utils_drop_get_uris		(GtkSelectionData *selection_data);
+
 G_END_DECLS
 
 #endif /* __GEDIT_UTILS_H__ */
