@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-view.c 5682 2007-07-03 09:41:20Z pborelli $ 
+ * $Id: gedit-view.c 5744 2007-07-26 10:11:16Z jessevdk $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -61,6 +61,11 @@ typedef enum
 	SEARCH
 } SearchMode;
 
+enum
+{
+	TARGET_URI_LIST = 100
+};
+
 struct _GeditViewPrivate
 {
 	GtkTooltips *tooltips;
@@ -97,25 +102,42 @@ struct _GeditViewPrivate
 /* The search entry completion is shared among all the views */
 GtkListStore *search_completion_model = NULL;
 
-static void	gedit_view_destroy		(GtkObject       *object);
-static void	gedit_view_finalize		(GObject         *object);
-static gint     gedit_view_focus_out		(GtkWidget       *widget,
-						 GdkEventFocus   *event);
+static void	gedit_view_destroy		(GtkObject        *object);
+static void	gedit_view_finalize		(GObject          *object);
+static gint     gedit_view_focus_out		(GtkWidget        *widget,
+						 GdkEventFocus    *event);
+static gboolean gedit_view_drag_motion		(GtkWidget        *widget,
+						 GdkDragContext   *context,
+						 gint              x,
+						 gint              y,
+						 guint             time);
+static void     gedit_view_drag_data_received   (GtkWidget        *widget,
+						 GdkDragContext   *context,
+						 gint              x,
+						 gint              y,
+						 GtkSelectionData *selection_data,
+						 guint             info,
+						 guint             time);
+static gboolean gedit_view_drag_drop		(GtkWidget        *widget,
+	      					 GdkDragContext   *context,
+	      					 gint              x,
+	      					 gint              y,
+	      					 guint             time);
 
-static gboolean start_interactive_search	(GeditView       *view);
-static gboolean start_interactive_goto_line	(GeditView       *view);
-static gboolean reset_searched_text		(GeditView       *view);
+static gboolean start_interactive_search	(GeditView        *view);
+static gboolean start_interactive_goto_line	(GeditView        *view);
+static gboolean reset_searched_text		(GeditView        *view);
 
-static void	hide_search_window 		(GeditView       *view,
-						 gboolean         cancel);
+static void	hide_search_window 		(GeditView        *view,
+						 gboolean          cancel);
 
 
-static gint	gedit_view_expose	 	(GtkWidget       *widget,
-						 GdkEventExpose  *event);
-static void 	search_highlight_updated_cb	(GeditDocument   *doc,
-						 GtkTextIter     *start,
-						 GtkTextIter     *end,
-						 GeditView       *view);
+static gint	gedit_view_expose	 	(GtkWidget        *widget,
+						 GdkEventExpose   *event);
+static void 	search_highlight_updated_cb	(GeditDocument    *doc,
+						 GtkTextIter      *start,
+						 GtkTextIter      *end,
+						 GeditView        *view);
 						 
 G_DEFINE_TYPE(GeditView, gedit_view, GTK_TYPE_SOURCE_VIEW)
 
@@ -125,6 +147,7 @@ enum
 	START_INTERACTIVE_SEARCH,
 	START_INTERACTIVE_GOTO_LINE,
 	RESET_SEARCHED_TEXT,
+	DROP_URIS,
 	LAST_SIGNAL
 };
 
@@ -160,6 +183,24 @@ gedit_view_class_init (GeditViewClass *klass)
 
 	widget_class->focus_out_event = gedit_view_focus_out;
 	widget_class->expose_event = gedit_view_expose;
+	
+	/*
+	 * Override the gtk_text_view_drag_motion and drag_drop
+	 * functions to get URIs
+	 *
+	 * If the mime type is text/uri-list, then we will accept
+	 * the potential drop, or request the data (depending on the
+	 * function).
+	 *
+	 * If the drag context has any other mime type, then pass the
+	 * information onto the GtkTextView's standard handlers.
+	 * (widget_class->function_name).
+	 *
+	 * See bug #89881 for details
+	 */
+	widget_class->drag_motion = gedit_view_drag_motion;
+	widget_class->drag_data_received = gedit_view_drag_data_received;
+	widget_class->drag_drop = gedit_view_drag_drop;
 
 	klass->start_interactive_search = start_interactive_search;
 	klass->start_interactive_goto_line = start_interactive_goto_line;
@@ -192,6 +233,24 @@ gedit_view_class_init (GeditViewClass *klass)
 			      gedit_marshal_BOOLEAN__NONE,
 			      G_TYPE_BOOLEAN, 0);		
 
+	/* A new signal DROP_URIS has been added to allow plugins to intercept
+	 * the default dnd behaviour of 'text/uri-list'. GeditView now handles
+	 * dnd in the default handlers of drag_drop, drag_motion and 
+	 * drag_data_received. The view emits drop_uris from drag_data_received
+	 * if valid uris have been dropped. Plugins should connect to 
+	 * drag_motion, drag_drop and drag_data_received to change this 
+	 * default behaviour. They should _NOT_ use this signal because this
+	 * will not prevent gedit from loading the uri
+	 */
+	view_signals[DROP_URIS] =
+    		g_signal_new ("drop_uris",
+		  	      G_TYPE_FROM_CLASS (object_class),
+		  	      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		  	      G_STRUCT_OFFSET (GeditViewClass, drop_uris),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__BOXED,
+			      G_TYPE_NONE, 1, G_TYPE_STRV);
+
 	g_type_class_add_private (klass, sizeof (GeditViewPrivate));
 	
 	binding_set = gtk_binding_set_by_class (klass);
@@ -218,7 +277,9 @@ scroll_to_cursor_on_init (GeditView *view)
 
 static void 
 gedit_view_init (GeditView *view)
-{	
+{
+	GtkTargetList *tl;
+	
 	gedit_debug (DEBUG_VIEW);
 	
 	view->priv = GEDIT_VIEW_GET_PRIVATE (view);
@@ -242,26 +303,6 @@ gedit_view_init (GeditView *view)
 		gedit_view_set_font (view, TRUE, NULL);
 	}
 
-	if (!gedit_prefs_manager_get_use_default_colors ())
-	{
-		GdkColor background;
-		GdkColor text;
-		GdkColor selection;
-		GdkColor sel_text;
-
-		background = gedit_prefs_manager_get_background_color ();
-		text = gedit_prefs_manager_get_text_color ();
-		selection = gedit_prefs_manager_get_selection_color ();
-		sel_text = gedit_prefs_manager_get_selected_text_color ();
-
-		gedit_view_set_colors (view,
-				       FALSE,
-				       &background,
-				       &text,
-				       &selection,
-				       &sel_text);
-	}	
-
 	g_object_set (G_OBJECT (view), 
 		      "wrap_mode", gedit_prefs_manager_get_wrap_mode (),
 		      "show_line_numbers", gedit_prefs_manager_get_display_line_numbers (),
@@ -270,8 +311,8 @@ gedit_view_init (GeditView *view)
 		      "insert_spaces_instead_of_tabs", gedit_prefs_manager_get_insert_spaces (),
 		      "show_margin", gedit_prefs_manager_get_display_right_margin (), 
 		      "margin", gedit_prefs_manager_get_right_margin_position (),
-		      "highlight_current_line", gedit_prefs_manager_get_highlight_current_line (), 
-		      "smart_home_end", GTK_SOURCE_SMART_HOME_END_AFTER, /* TODO: add a gconf setting */
+		      "highlight_current_line", gedit_prefs_manager_get_highlight_current_line (),
+		      "smart_home_end", gedit_prefs_manager_get_smart_home_end (),
 		      "indent_on_tab", TRUE,
 		      NULL);
 
@@ -285,7 +326,13 @@ gedit_view_init (GeditView *view)
 	view->priv->scroll_idle = g_idle_add ((GSourceFunc) scroll_to_cursor_on_init, view);
 	
 	view->priv->typeselect_flush_timeout = 0;	
-	view->priv->wrap_around = TRUE;	
+	view->priv->wrap_around = TRUE;
+
+	/* Drag and drop support */	
+	tl = gtk_drag_dest_get_target_list (GTK_WIDGET (view));
+
+	if (tl != NULL)
+		gtk_target_list_add_uri_targets (tl, TARGET_URI_LIST);
 }
 
 static void
@@ -536,143 +583,6 @@ gedit_view_scroll_to_cursor (GeditView *view)
 				      0.0);
 }
 
-/* assign a unique name */
-static G_CONST_RETURN gchar *
-get_widget_name (GtkWidget *w)
-{
-	const gchar *name;	
-
-	name = gtk_widget_get_name (w);
-	g_return_val_if_fail (name != NULL, NULL);
-
-	if (strcmp (name, g_type_name (GTK_WIDGET_TYPE (w))) == 0)
-	{
-		static guint d = 0;
-		gchar *n;
-
-		n = g_strdup_printf ("%s_%u_%u", name, d, (guint) g_random_int);
-		d++;
-
-		gtk_widget_set_name (w, n);
-		g_free (n);
-
-		name = gtk_widget_get_name (w);
-	}
-
-	return name;
-}
-
-/* There is no clean way to set the cursor-color, so we are stuck
- * with the following hack: set the name of each widget and parse
- * a gtkrc string.
- */
-static void 
-modify_cursor_color (GtkWidget *textview, 
-		     GdkColor  *color)
-{
-	static const char cursor_color_rc[] =
-		"style \"svs-cc\"\n"
-		"{\n"
-			"GtkSourceView::cursor-color=\"#%04x%04x%04x\"\n"
-		"}\n"
-		"widget \"*.%s\" style : application \"svs-cc\"\n";
-
-	const gchar *name;
-	gchar *rc_temp;
-
-	gedit_debug (DEBUG_VIEW);
-
-	name = get_widget_name (textview);
-	g_return_if_fail (name != NULL);
-
-	if (color != NULL)
-	{
-		rc_temp = g_strdup_printf (cursor_color_rc,
-					   color->red, 
-					   color->green, 
-					   color->blue,
-					   name);
-	}
-	else
-	{
-		GtkRcStyle *rc_style;
-
- 		rc_style = gtk_widget_get_modifier_style (textview);
-
-		rc_temp = g_strdup_printf (cursor_color_rc,
-					   rc_style->text [GTK_STATE_NORMAL].red,
-					   rc_style->text [GTK_STATE_NORMAL].green,
-					   rc_style->text [GTK_STATE_NORMAL].blue,
-					   name);
-	}
-
-	gtk_rc_parse_string (rc_temp);
-	gtk_widget_reset_rc_styles (textview);
-
-	g_free (rc_temp);
-}
-
-void
-gedit_view_set_colors (GeditView *view,
-		       gboolean   def,
-		       GdkColor  *backgroud,
-		       GdkColor  *text,
-		       GdkColor  *selection,
-		       GdkColor  *sel_text)
-{
-	gedit_debug (DEBUG_VIEW);
-
-	g_return_if_fail (GEDIT_IS_VIEW (view));
-
-	/* just a bit of paranoia */
-	gtk_widget_ensure_style (GTK_WIDGET (view));
-
-	if (!def)
-	{
-		if (backgroud != NULL)
-			gtk_widget_modify_base (GTK_WIDGET (view), 
-						GTK_STATE_NORMAL, backgroud);
-
-		if (selection != NULL)
-		{
-			gtk_widget_modify_base (GTK_WIDGET (view), 
-						GTK_STATE_SELECTED, selection);
-			gtk_widget_modify_base (GTK_WIDGET (view), 
-						GTK_STATE_ACTIVE, selection);
-		}
-
-		if (sel_text != NULL)
-		{
-			gtk_widget_modify_text (GTK_WIDGET (view), 
-						GTK_STATE_SELECTED, sel_text);		
-			gtk_widget_modify_text (GTK_WIDGET (view), 
-						GTK_STATE_ACTIVE, sel_text);		
-		}
-
-		if (text != NULL)
-		{
-			gtk_widget_modify_text (GTK_WIDGET (view), 
-						GTK_STATE_NORMAL, text);
-			modify_cursor_color (GTK_WIDGET (view), text);
-		}
-	}
-	else
-	{
-		GtkRcStyle *rc_style;
-
-		rc_style = gtk_widget_get_modifier_style (GTK_WIDGET (view));
-
-		rc_style->color_flags [GTK_STATE_NORMAL] = 0;
-		rc_style->color_flags [GTK_STATE_SELECTED] = 0;
-		rc_style->color_flags [GTK_STATE_ACTIVE] = 0;
-
-		gtk_widget_modify_style (GTK_WIDGET (view), rc_style);
-
-		/* It must be called after the text color has been modified */
-		modify_cursor_color (GTK_WIDGET (view), NULL);
-	}
-}
-
 void
 gedit_view_set_font (GeditView   *view, 
 		     gboolean     def, 
@@ -1794,6 +1704,103 @@ gedit_view_expose (GtkWidget      *widget,
 	return (* GTK_WIDGET_CLASS (gedit_view_parent_class)->expose_event)(widget, event);
 }
 
+static GdkAtom
+drag_get_uri_target (GtkWidget      *widget,
+		     GdkDragContext *context)
+{
+	GdkAtom target;
+	GtkTargetList *tl;
+	
+	tl = gtk_target_list_new (NULL, 0);
+	gtk_target_list_add_uri_targets (tl, 0);
+	
+	target = gtk_drag_dest_find_target (widget, context, tl);
+	gtk_target_list_unref (tl);
+	
+	return target;	
+}
+
+static gboolean
+gedit_view_drag_motion (GtkWidget      *widget,
+			GdkDragContext *context,
+			gint            x,
+			gint            y,
+			guint           time)
+{
+	gboolean result;
+
+	/* Chain up to allow textview to scroll and position dnd mark, note 
+	 * that this needs to be checked if gtksourceview or gtktextview
+	 * changes drag_motion behaviour */
+	result = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_motion (widget, context, x, y, time);
+
+	/* If this is a URL, deal with it here */
+	if (drag_get_uri_target (widget, context) != GDK_NONE) 
+	{
+		gdk_drag_status (context, context->suggested_action, time);
+		result = TRUE;
+	}
+
+	return result;
+}
+
+static void
+gedit_view_drag_data_received (GtkWidget        *widget,
+		       	       GdkDragContext   *context,
+			       gint              x,
+			       gint              y,
+			       GtkSelectionData *selection_data,
+			       guint             info,
+			       guint             time)
+{
+	gchar **uri_list;
+	
+	/* If this is an URL emit DROP_URIS, otherwise chain up the signal */
+	if (info == TARGET_URI_LIST)
+	{
+		uri_list = gedit_utils_drop_get_uris (selection_data);
+		
+		if (uri_list != NULL)
+		{
+			g_signal_emit (widget, view_signals[DROP_URIS], 0, uri_list);
+			g_strfreev (uri_list);
+			
+			gtk_drag_finish (context, TRUE, FALSE, time);
+		}
+	}
+	else
+	{
+		GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_data_received (widget, context, x, y, selection_data, info, time);
+	}
+}
+
+static gboolean
+gedit_view_drag_drop (GtkWidget      *widget,
+		      GdkDragContext *context,
+		      gint            x,
+		      gint            y,
+		      guint           time)
+{
+	gboolean result;
+	GdkAtom target;
+
+	/* If this is a URL, just get the drag data */
+	target = drag_get_uri_target (widget, context);
+
+	if (target != GDK_NONE)
+	{
+		gtk_drag_get_data (widget, context, target, time);
+		result = TRUE;
+	}
+	else
+	{
+		/* Chain up */
+		result = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_drop (widget, context, x, y, time);
+	}
+
+	return result;
+}
+
 static void 	
 search_highlight_updated_cb (GeditDocument *doc,
 			     GtkTextIter   *start,
