@@ -55,11 +55,11 @@ typedef struct _GeditViewPrivate	GeditViewPrivate;
  * Main object structure
  */
 typedef struct _GeditView		GeditView;
- 
+
 struct _GeditView
 {
 	GtkSourceView view;
-	
+
 	/*< private > */
 	GeditViewPrivate *priv;
 };
@@ -68,14 +68,19 @@ struct _GeditView
  * Class definition
  */
 typedef struct _GeditViewClass		GeditViewClass;
- 
+
 struct _GeditViewClass
 {
 	GtkSourceViewClass parent_class;
-	
+
 	/* FIXME: Do we need placeholders ? */
+
+	/* Key bindings */
 	gboolean (* start_interactive_search)	(GeditView       *view);
 	gboolean (* start_interactive_goto_line)(GeditView       *view);	
+	gboolean (* reset_searched_text)	(GeditView       *view);
+	void	 (* drop_uris)			(GeditView	 *view, 
+						 gchar          **uri_list);
 };
 
 /*
@@ -93,13 +98,6 @@ void		 gedit_view_select_all		(GeditView       *view);
 
 void		 gedit_view_scroll_to_cursor 	(GeditView       *view);
 
-void 		 gedit_view_set_colors 		(GeditView       *view, 
-						 gboolean         def,
-						 GdkColor        *backgroud, 
-						 GdkColor        *text,
-						 GdkColor        *selection, 
-						 GdkColor        *sel_text);
-
 void 		 gedit_view_set_font		(GeditView       *view,
 						 gboolean         def,
 						 const gchar     *font_name);
