@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c 5646 2007-06-24 19:56:01Z pborelli $
+ * $Id: gedit-window.c 5750 2007-07-27 10:40:00Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -83,11 +83,6 @@ enum
 	TARGET_URI_LIST = 100
 };
 
-static const GtkTargetEntry drag_types[] =
-{
-	{ "text/uri-list", 0, TARGET_URI_LIST },
-};
-
 G_DEFINE_TYPE(GeditWindow, gedit_window, GTK_TYPE_WINDOW)
 
 static void	recent_manager_changed	(GtkRecentManager *manager,
@@ -799,14 +794,15 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 	gedit_plugins_engine_update_plugins_ui (window, FALSE);
 }
 
+#define GEDIT_SOURCE_LANGUAGE_KEY "sourcelanguage"
+
 static void
 language_toggled (GtkToggleAction *action,
 		  GeditWindow     *window)
 {
 	GeditDocument *doc;
-	const GSList *languages;
-	const GtkSourceLanguage *lang;
-	gint n;
+	GtkSourceLanguage *lang;
+	const gchar *lang_id;
 
 	if (gtk_toggle_action_get_active (action) == FALSE)
 		return;
@@ -815,22 +811,25 @@ language_toggled (GtkToggleAction *action,
 	if (doc == NULL)
 		return;
 
-	n = gtk_radio_action_get_current_value (GTK_RADIO_ACTION (action));
-
-	if (n < 0)
+	lang_id = g_object_get_data (G_OBJECT (action),
+				     GEDIT_SOURCE_LANGUAGE_KEY);
+	if (lang_id == NULL)
 	{
 		/* Normal (no highlighting) */
 		lang = NULL;
 	}
 	else
 	{
-		languages = gedit_language_manager_get_available_languages_sorted (
-						gedit_get_language_manager ());
-
-		lang = g_slist_nth_data ((GSList *) languages, n);
+		lang = gtk_source_language_manager_get_language_by_id (
+				gedit_get_language_manager (),
+				lang_id);
+		if (lang == NULL)
+		{
+			g_warning ("Could not get language %s\n", lang_id);
+		}
 	}
 
-	gedit_document_set_language (doc, (GtkSourceLanguage *) lang);
+	gedit_document_set_language (doc, lang);
 }
 
 static gchar *
@@ -863,6 +862,7 @@ create_language_menu_item (GtkSourceLanguage *lang,
 	GSList *group;
 	const gchar *section;
 	gchar *escaped_section;
+	const gchar *lang_id;
 	const gchar *lang_name;
 	gchar *escaped_lang_name;
 	gchar *tip;
@@ -908,6 +908,13 @@ create_language_menu_item (GtkSourceLanguage *lang,
 				       NULL,
 				       index);
 
+	/* store lang id */
+	lang_id = gtk_source_language_get_id (lang);
+	g_object_set_data_full (G_OBJECT (action),
+				GEDIT_SOURCE_LANGUAGE_KEY,
+				g_strdup (lang_id),
+				(GDestroyNotify) g_free);
+
 	/* Action is added with a NULL accel to make the accel overridable */
 	gtk_action_group_add_action_with_accel (window->priv->languages_action_group,
 						GTK_ACTION (action),
@@ -942,8 +949,8 @@ static void
 create_languages_menu (GeditWindow *window)
 {
 	GtkRadioAction *action_none;
-	const GSList *languages;
-	const GSList *l;
+	GSList *languages;
+	GSList *l;
 	guint id;
 	gint i;
 
@@ -979,14 +986,19 @@ create_languages_menu (GeditWindow *window)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_none), TRUE);
 
 	/* now add all the known languages */
-	languages = gedit_language_manager_get_available_languages_sorted (
-						gedit_get_language_manager ());
+	languages = gedit_language_manager_list_languages_sorted (
+						gedit_get_language_manager (),
+						FALSE);
 
 	for (l = languages, i = 0; l != NULL; l = l->next, ++i)
+	{
 		create_language_menu_item (l->data,
-					    i,
-					    id,
-					    window);
+					   i,
+					   id,
+					   window);
+	}
+
+	g_slist_free (languages);
 }
 
 static void
@@ -2263,6 +2275,45 @@ sync_name (GeditTab    *tab,
 	gedit_plugins_engine_update_plugins_ui (window, FALSE);
 }
 
+static GeditWindow *
+get_drop_window (GtkWidget *widget)
+{
+	GtkWidget *target_window;
+
+	target_window = gtk_widget_get_toplevel (widget);
+	g_return_val_if_fail (GEDIT_IS_WINDOW (target_window), NULL);
+
+	if ((GEDIT_WINDOW(target_window)->priv->state & GEDIT_WINDOW_STATE_SAVING_SESSION) != 0)
+		return NULL;
+	
+	return GEDIT_WINDOW (target_window);
+}
+
+static void
+load_uris_from_drop (GeditWindow  *window,
+		     gchar       **uri_list)
+{
+	GSList *uris = NULL;
+	gint i;
+	
+	if (uri_list == NULL)
+		return;
+	
+	for (i = 0; uri_list[i] != NULL; ++i)
+	{
+		uris = g_slist_prepend (uris, uri_list[i]);
+	}
+
+	uris = g_slist_reverse (uris);
+	gedit_commands_load_uris (window,
+				  uris,
+				  NULL,
+				  0);
+
+	g_slist_free (uris);
+}
+
+/* Handle drops on the GeditWindow */
 static void
 drag_data_received_cb (GtkWidget        *widget,
 		       GdkDragContext   *context,
@@ -2273,128 +2324,35 @@ drag_data_received_cb (GtkWidget        *widget,
 		       guint             time,
 		       gpointer          data)
 {
-	GtkWidget *target_window;
-	gchar **uris;
-	GSList *uri_list = NULL;
-	gint i;
+	GeditWindow *window;
+	gchar **uri_list;
 
-	if (info != TARGET_URI_LIST)
-		return;
-
-	g_return_if_fail (widget != NULL);
-
-	target_window = gtk_widget_get_toplevel (widget);
-	g_return_if_fail (GEDIT_IS_WINDOW (target_window));
+	window = get_drop_window (widget);
 	
-	if ((GEDIT_WINDOW(target_window)->priv->state & GEDIT_WINDOW_STATE_SAVING_SESSION) != 0)
+	if (window == NULL)
 		return;
 
-	uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
-
-	for (i = 0; uris[i] != NULL; i++)
+	if (info == TARGET_URI_LIST)
 	{
-		gchar *uri;
-		
-		uri = gedit_utils_make_canonical_uri_from_shell_arg (uris[i]);
-		
-		/* Silently ignore malformed URI/filename */
-		if (uri != NULL)
-			uri_list = g_slist_prepend (uri_list, uri);	
+		uri_list = gedit_utils_drop_get_uris(selection_data);
+		load_uris_from_drop (window, uri_list);
+		g_strfreev (uri_list);
 	}
+}
 
-	g_strfreev (uris);
+/* Handle drops on the GeditView */
+static void
+drop_uris_cb (GtkWidget    *widget,
+	      gchar       **uri_list)
+{
+	GeditWindow *window;
 
-	if (uri_list == NULL)
+	window = get_drop_window (widget);
+	
+	if (window == NULL)
 		return;
 
-	uri_list = g_slist_reverse (uri_list);
-
-	gedit_commands_load_uris (GEDIT_WINDOW (target_window),
-				  uri_list,
-				  NULL,
-				  0);
-
-	g_slist_foreach (uri_list, (GFunc) g_free, NULL);	
-	g_slist_free (uri_list);
-}
-
-/*
- * Override the gtk_text_view_drag_motion and drag_drop 
- * functions to get URIs
- *
- * If the mime type is text/uri-list, then we will accept
- * the potential drop, or request the data (depending on the
- * function).
- *
- * If the drag context has any other mime type, then pass the
- * information onto the GtkTextView's standard handlers.
- * (widget_class->function_name).
- *
- * See bug #89881 for details
- */
-
-static gboolean
-drag_motion_cb (GtkWidget      *widget,
-		GdkDragContext *context,
-		gint            x,
-		gint            y,
-		guint           time)
-{
-	GtkTargetList *tl;
-	GtkWidgetClass *widget_class;
-	gboolean result;
-
-	tl = gtk_target_list_new (drag_types,
-				  G_N_ELEMENTS (drag_types));
-
-	/* If this is a URL, deal with it here, or pass to the text view */
-	if (gtk_drag_dest_find_target (widget, context, tl) != GDK_NONE) 
-	{
-		gdk_drag_status (context, context->suggested_action, time);
-		result = TRUE;
-	}
-	else
-	{
-		widget_class = GTK_WIDGET_GET_CLASS (widget);
-		result = (*widget_class->drag_motion) (widget, context, x, y, time);
-	}
-
-	gtk_target_list_unref (tl);
-
-	return result;
-}
-
-static gboolean
-drag_drop_cb (GtkWidget      *widget,
-	      GdkDragContext *context,
-	      gint            x,
-	      gint            y,
-	      guint           time)
-{
-	GtkTargetList *tl;
-	GtkWidgetClass *widget_class;
-	gboolean result;
-	GdkAtom target;
-
-	tl = gtk_target_list_new (drag_types,
-				  G_N_ELEMENTS (drag_types));
-
-	/* If this is a URL, just get the drag data */
-	target = gtk_drag_dest_find_target (widget, context, tl);
-	if (target != GDK_NONE)
-	{
-		gtk_drag_get_data (widget, context, target, time);
-		result = TRUE;
-	}
-	else
-	{
-		widget_class = GTK_WIDGET_GET_CLASS (widget);
-		result = (*widget_class->drag_drop) (widget, context, x, y, time);
-	}
-
-	gtk_target_list_unref (tl);
-
-	return result;
+	load_uris_from_drop (window, uri_list);
 }
 
 static void
@@ -2505,76 +2463,6 @@ sync_languages_menu (GeditDocument *doc,
 	update_languages_menu (window);
 }
 
-static void
-update_default_path (GeditWindow   *window,
-		     GeditDocument *doc)
-{
-	gchar *uri;
-
-	uri = gedit_document_get_uri (doc);
-	// CHECK: what does it happens when loading from stdin? - Paolo
-	g_return_if_fail (uri != NULL);
-
-	if (gedit_utils_uri_has_file_scheme (uri))
-	{
-		gchar *default_path;
-
-		// CHECK: does it work with uri chaining? - Paolo
-		default_path = g_path_get_dirname (uri);
-
-		g_return_if_fail (strlen (default_path) >= 5 /* strlen ("file:") */);
-		if (strcmp (default_path, "file:") == 0)
-		{
-			g_free (default_path);
-		
-			default_path = g_strdup ("file:///");
-		}
-
-		g_free (window->priv->default_path);
-
-		window->priv->default_path = default_path;
-
-		gedit_debug_message (DEBUG_WINDOW, 
-				     "New default path: %s", default_path);
-	}
-
-	g_free (uri);
-}
-
-static void
-doc_loaded (GeditDocument *doc,
-	    const GError  *error,
-	    GeditWindow   *window)
-{
-	if (error != NULL)
-	{
-		gedit_debug_message (DEBUG_WINDOW, "Error");
-		return;
-	}
-	
-	update_default_path (window, doc);
-}
-
-static void
-doc_saved (GeditDocument *doc,
-	   const GError  *error,
-	   GeditWindow   *window)
-{
-	if (error != NULL)
-	{
-		gedit_debug_message (DEBUG_WINDOW, "Error");
-		return;
-	}
-	
-	if (!_gedit_document_is_saving_as (doc))
-	{
-		gedit_debug_message (DEBUG_WINDOW, "Not saving as");
-		return;
-	}
-
-	update_default_path (window, doc);
-}
-
 static void
 editable_changed (GeditView  *view,
                   GParamSpec  *arg1,
@@ -2590,7 +2478,6 @@ notebook_tab_added (GeditNotebook *notebook,
 {
 	GeditView *view;
 	GeditDocument *doc;
-	GtkTargetList *tl;
 	GtkAction *action;
 
 	gedit_debug (DEBUG_WINDOW);
@@ -2648,14 +2535,6 @@ notebook_tab_added (GeditNotebook *notebook,
 			  "notify::language",
 			  G_CALLBACK (sync_languages_menu),
 			  window);
-	g_signal_connect (doc,
-			  "loaded",
-			  G_CALLBACK (doc_loaded),
-			  window);			  			  
-	g_signal_connect (doc,
-			  "saved",
-			  G_CALLBACK (doc_saved),
-			  window);
 	g_signal_connect (view,
 			  "toggle_overwrite",
 			  G_CALLBACK (update_overwrite_mode_statusbar),
@@ -2667,29 +2546,9 @@ notebook_tab_added (GeditNotebook *notebook,
 
 	update_documents_list_menu (window);
 	
-	/* CHECK: it seems to me this does not work when tab are moved between
-	   windows */
-	/* Drag and drop support */
-	tl = gtk_drag_dest_get_target_list (GTK_WIDGET (view));
-	g_return_if_fail (tl != NULL);
-
-	gtk_target_list_add_table (tl, drag_types, G_N_ELEMENTS (drag_types));
-
 	g_signal_connect (view,
-			  "drag_data_received",
-			  G_CALLBACK (drag_data_received_cb), 
-			  NULL);
-
-	/* Get signals before the standard text view functions to deal 
-	 * with uris for text files.
-	 */
-	g_signal_connect (view,
-			  "drag_motion",
-			  G_CALLBACK (drag_motion_cb), 
-			  NULL);
-	g_signal_connect (view,
-			  "drag_drop",
-			  G_CALLBACK (drag_drop_cb), 
+			  "drop_uris",
+			  G_CALLBACK (drop_uris_cb), 
 			  NULL);
 
 	update_window_state (window);
@@ -2705,16 +2564,16 @@ notebook_tab_removed (GeditNotebook *notebook,
 	GeditView     *view;
 	GeditDocument *doc;
 	GtkAction     *action;
-	
+
 	gedit_debug (DEBUG_WINDOW);
-	
+
 	g_return_if_fail ((window->priv->state & GEDIT_WINDOW_STATE_SAVING_SESSION) == 0);
-		
+
 	--window->priv->num_tabs;
-	
+
 	view = gedit_tab_get_view (tab);
 	doc = gedit_tab_get_document (tab);
-	
+
 	g_signal_handlers_disconnect_by_func (tab,
 					      G_CALLBACK (sync_name), 
 					      window);
@@ -2739,12 +2598,6 @@ notebook_tab_removed (GeditNotebook *notebook,
 	g_signal_handlers_disconnect_by_func (doc,
 					      G_CALLBACK (sync_languages_menu),
 					      window);
-	g_signal_handlers_disconnect_by_func (doc, 
-					      G_CALLBACK (doc_loaded),
-					      window);
-	g_signal_handlers_disconnect_by_func (doc, 
-					      G_CALLBACK (doc_saved),
-					      window);					      				      
 	g_signal_handlers_disconnect_by_func (view, 
 					      G_CALLBACK (update_overwrite_mode_statusbar),
 					      window);
@@ -2752,13 +2605,7 @@ notebook_tab_removed (GeditNotebook *notebook,
 					      G_CALLBACK (editable_changed),
 					      window);
 	g_signal_handlers_disconnect_by_func (view, 
-					      G_CALLBACK (drag_data_received_cb),
-					      NULL);
-	g_signal_handlers_disconnect_by_func (view, 
-					      G_CALLBACK (drag_motion_cb),
-					      NULL);
-	g_signal_handlers_disconnect_by_func (view, 
-					      G_CALLBACK (drag_drop_cb),
+					      G_CALLBACK (drop_uris_cb),
 					      NULL);
 
 	g_return_if_fail (window->priv->num_tabs >= 0);
@@ -3215,6 +3062,7 @@ gedit_window_init (GeditWindow *window)
 {
 	static gboolean is_first = TRUE;
 	GtkWidget *main_box;
+	GtkTargetList *tl;
 
 	gedit_debug (DEBUG_WINDOW);
 
@@ -3287,15 +3135,28 @@ gedit_window_init (GeditWindow *window)
 	gtk_widget_show (window->priv->hpaned);
 	gtk_widget_show (window->priv->vpaned);
 
-	/* Drag and drop support */
+	/* Drag and drop support, set targets to NULL because we add the
+	   default uri_targets below */
 	gtk_drag_dest_set (GTK_WIDGET (window),
 			   GTK_DEST_DEFAULT_MOTION |
 			   GTK_DEST_DEFAULT_HIGHLIGHT |
 			   GTK_DEST_DEFAULT_DROP,
-			   drag_types,
-			   G_N_ELEMENTS (drag_types),
+			   NULL,
+			   0,
 			   GDK_ACTION_COPY);
 
+	/* Add uri targets */
+	tl = gtk_drag_dest_get_target_list (GTK_WIDGET (window));
+	
+	if (tl == NULL)
+	{
+		tl = gtk_target_list_new (NULL, 0);
+		gtk_drag_dest_set_target_list (GTK_WIDGET (window), tl);
+		gtk_target_list_unref (tl);
+	}
+	
+	gtk_target_list_add_uri_targets (tl, TARGET_URI_LIST);
+
 	/* Connect signals */
 	g_signal_connect (window->priv->notebook,
 			  "switch_page",
@@ -3666,6 +3527,36 @@ _gedit_window_get_default_path (GeditWindow *window)
 	return window->priv->default_path;
 }
 
+void
+_gedit_window_set_default_path (GeditWindow *window,
+				const gchar *uri)
+{
+	g_return_if_fail (uri != NULL);
+
+	if (gedit_utils_uri_has_file_scheme (uri))
+	{
+		gchar *default_path;
+
+		// CHECK: does it work with uri chaining? - Paolo
+		default_path = g_path_get_dirname (uri);
+
+		g_return_if_fail (strlen (default_path) >= 5 /* strlen ("file:") */);
+		if (strcmp (default_path, "file:") == 0)
+		{
+			g_free (default_path);
+		
+			default_path = g_strdup ("file:///");
+		}
+
+		g_free (window->priv->default_path);
+
+		window->priv->default_path = default_path;
+
+		gedit_debug_message (DEBUG_WINDOW, 
+				     "New default path: %s", default_path);
+	}
+}
+
 /* Returns the documents that need to be saved before closing the window */
 GList *
 gedit_window_get_unsaved_documents (GeditWindow *window)
